﻿
'■ユニット
'主人公やビーム、ボムなどを「ユニット」と呼んでいます。
'背景や、得点表示、残り体力などの表示もユニットです。
'プログラム的にはUnitクラスを継承して作成され、Timer1_TickでMoveとDrawを呼び出すので、
'Unitごとに自分で座標を算出して、自分で自分を描画します。

'■フレーム
'Timer1_Tickが１回呼ばれることを「１フレーム」と呼んでいます。
'このゲームでの時間の単位になります。
'たとえば、通常のジャンプは開始から終了まで30フレームかかります。(→Characterクラスで設定)

'■その他
'ソリューションエクスプローラにReadme.txtがあるので読んでください。

Public Class Form1

    Private Input As InputState     'キーボードの入力を操作と関連付ける。例：Shift→ジャンプ
    Private Units As New UnitCollection
    Private mainGraphics As Graphics    '画面描画用のGraphicsクラス。
    Private Score As Counter '得点表示領域
    Private HiScore As Counter  '最高得点表示領域
    Private hiscoreValue As Integer
    Private RestPlayers As Counter  'プレイヤー残機数
    Private debugMessage As Message
    Private BackGround As Background

    ''' <summary>
    ''' これがTrueのときStoryBoardの読み込みはされず、敵のでてこない空間をスクロールして航行する。
    ''' これはプレイヤー機が撃墜され、再スタートする場合に使用する機能。
    ''' </summary>
    ''' <remarks></remarks>
    Private IsMoratorium As Boolean

    ''' <summary>
    ''' UnitAddRequestイベントで追加を依頼されたユニットの一覧。
    ''' ループごとに生成。
    ''' このコレクションに追加されたユニットは画面に出現して活動を開始する。
    ''' </summary>
    ''' <remarks></remarks>
    Private requestedUnits As New List(Of UnitBase)

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Init()

    End Sub

    Private Sub Init()


        Units.Clear()
        requestedUnits.Clear()
        RemoveHandler Common.UnitAddRequested, AddressOf UnitAddRequested
        AddHandler Common.UnitAddRequested, AddressOf UnitAddRequested
        Common.GamePhase = GamePhases.Playing

        '▼プレイ領域の定義
        '領域外にも点数やメッセージの表示はあり得る
        Common.Display = New DisplayInfo(New Rectangle(0, 0, Me.ClientRectangle.Width, Me.ClientRectangle.Height))

        '▼主人公キャラのユニット生成
        Common.mainUnit = New StarA()
        Common.mainUnit.Left = (Common.Display.ViewArea.Width - Common.mainUnit.Width) \ 2
        Common.mainUnit.Top = Common.Display.ViewArea.Bottom - 80

        'デバッグ用(主人公機のパワーアップ段階)
        'Common.mainUnit.PowerUpLevel = 10

        Units.Add(Common.mainUnit)

        '▼背景の生成(背景はユニットっぽくないがここでは巨大なユニット扱い)
        BackGround = New Background()

        '▼各種表示領域(これもユニットっぽくないがここでは動かないユニット扱い)

        'スコア(得点)
        Score = New Counter("SCORE", 300, 1, 9)
        Units.Add(Score)

        'ハイスコア(今までの最高得点)
        HiScore = New Counter("HISCORE", 1, 1, 9)
        HiScore.Value = hiscoreValue
        Units.Add(HiScore)

        'プレイヤー残機数
        RestPlayers = New Counter("PLAYERS", 600, 1, 2)
        RestPlayers.Value = 3
        Units.Add(RestPlayers)


        '▼画面下部のでバグ用メッセージ領域の生成(念のため言っておくと、これもユニット)
        debugMessage = New Message("", 10, Common.Display.ViewArea.Bottom - 40, Color.Gray)
        'debugMessage.Text = "debugMessage.Text = ""…"" でここにメッセージを表示できます。"
        Units.Add(debugMessage)

        '▼描画用のGraphicsクラスの確保(技術的な処理)
        If mainGraphics Is Nothing Then
            '初回のみ生成
            Dim bmp As New Bitmap(Me.ClientRectangle.Width, Me.ClientRectangle.Height)
            Me.BackgroundImage = bmp
            mainGraphics = Graphics.FromImage(bmp)
        End If

        Input = New InputState
        Common.StoryBoard = New StoryBoard


        Me.Show()
        Me.Focus()  '入力が受け取れるようにする。最初はなくてもよいがリトライ時必須

        SoundComposer.BGM1.Play()
        timerMain.Enabled = True
    End Sub

   
    Private Sub Form1_KeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown

        If e.KeyCode = Keys.Escape Then
            timerMain.Enabled = Not timerMain.Enabled
        End If

        Input.Input(e, True)

    End Sub

    Private Sub Form1_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyUp

        Input.Input(e, False)

    End Sub

    'Private KeepTime As Boolean

    Private Sub timerMain_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles timerMain.Tick

        'If KeepTime Then
        '    Return
        'End If
        'KeepTime = True

        If Input.Start Then
            If Common.GamePhase = GamePhases.Gameover Then
                timerMain.Enabled = False
                Init()
                Return
            End If
        End If


        '▼前処理

        '画面を黒でクリア(この時点ではすべてのユニットが画面から消える。)
        'なお、実際に画面に反映されるのは下のほうにあるMe.Invalidateを呼び出したときで、
        'それまでは、何を書いても画面には反映されない。
        mainGraphics.Clear(Color.Black)

        requestedUnits.Clear()

        'debugMessage.Text = Input.Fire & " - " & Input.DebugMessage

        '▼敵生成

        If Not IsMoratorium Then
            Dim newUnit As UnitBase = Common.StoryBoard.Tick

            If newUnit IsNot Nothing Then
                Units.Add(newUnit)
            End If
        End If


        BackGround.Draw(mainGraphics)

        '▼各ユニットの移動と描画

        '今回消滅するユニットの一覧
        '消滅は敵ユニットが画面の左端までいったときや主人公と衝突したときに発生
        Dim DeadUnits As New List(Of UnitBase)

        For Each unit As UnitBase In Units

            'ユニット消滅判定(ここでは左端に行ったユニットが消滅ユニット一覧に追加される。
            If unit.IsDead Then
                DeadUnits.Add(unit)
                Continue For
            End If

            'ユニットの移動と描画
            unit.Move(Input)
            unit.Draw(mainGraphics)

        Next

        '▼当たり判定
        For Each enemy As UnitBase In Units.Enemies
            For Each ally As UnitBase In Units.Allies

                '弾同士の衝突は何もしない
                If enemy.IsFire AndAlso ally.IsFire Then
                    Exit For
                End If


                If IsHit(enemy, ally) Then

                    If ally.AttackPriority <= enemy.AttackPriority Then

                        If ally.Damage(enemy.AttackPoint) = False Then
                            DeadUnits.Add(ally)
                            ally.Explode()

                            '撃墜されたのが主人公機の場合
                            If ally Is Common.mainUnit Then

                                If RestPlayers.Value = 0 Then
                                    GameOver()
                                Else
                                    Restart()
                                End If

                            End If
                        End If

                    End If

                    If enemy.AttackPriority <= ally.AttackPriority Then

                        If enemy.Damage(ally.AttackPoint) = False Then
                            DeadUnits.Add(enemy)
                            enemy.Explode()
                            Score.Value += enemy.GamePoint
                            'GoTo nextenemy
                            Exit For
                        End If

                    End If

                End If
            Next
            'NextEnemy:
        Next

        'アイテム取得判定
        For Each unit As UnitBase In Units.Items
            If IsHit(unit, Common.mainUnit) Then
                DeadUnits.Add(unit)
                'この音楽を鳴らす処理はPowerupクラスの方に実装したい。
                SoundComposer.PowerUp.Play()
                Common.mainUnit.PowerUpLevel += 1
            End If
        Next


        For Each unit As UnitBase In requestedUnits
            Units.Add(unit)
        Next

        '消滅したユニットを一覧から削除
        For Each unit As UnitBase In DeadUnits
            Units.Remove(unit)
        Next




        '▼画面更新
        Me.Invalidate()

        'KeepTime = False

    End Sub


    ''' <summary>
    ''' 当たり判定。unit1とunit2に重なっている部分があるときTrue
    ''' </summary>
    Private Function IsHit(ByVal unit1 As UnitBase, ByVal unit2 As UnitBase) As Boolean

        Dim cloneRect As Rectangle = unit1.Rectangle

        cloneRect.Intersect(unit2.Rectangle)

        If cloneRect.Width > 0 Then
            Return True
        End If

    End Function

    Private Sub UnitAddRequested(ByVal unit As UnitBase)
        requestedUnits.Add(unit)
    End Sub

    ''' <summary>プレイヤー機撃墜時に残機がないとき</summary>
    Private Sub GameOver()
        SoundComposer.BGM1.Stop()
        SoundComposer.Gameover.Play()

        Dim message1 As New Message("GAME OVER", (Common.Display.ViewArea.Width - 400) \ 2, (Common.Display.ViewArea.Height - 100) \ 2, Color.Red, 64)
        requestedUnits.Add(message1)

        Dim message2 As New Message("Enterキーで再開できます。", message1.Left, message1.Top + 100, Color.Red)
        requestedUnits.Add(message2)

        If hiscoreValue < Score.Value Then
            hiscoreValue = Score.Value
        End If

        Common.GamePhase = GamePhases.Gameover

    End Sub

    ''' <summary>プレイヤー機撃墜時に残機があるとき</summary>
    Private Sub Restart()
        SoundComposer.BGM1.Stop()
        timerRestart.Enabled = True
    End Sub

    Private restartTimerTickCount As Integer
    Private Sub timerRestart_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles timerRestart.Tick

        restartTimerTickCount += 1


        'プレイヤー機撃墜時の流れ(→ゲームオーバー時は別の流れ)
        '0秒後～5秒後    プレイヤー機は消えるが、スクロールは続き、敵機も出現する。
        '5秒後～10秒後   5秒後に第１回Tick発生。

        Select Case restartTimerTickCount
            Case 1  '第１回Tick
                '残存敵を消滅させる。
                For Each unit As UnitBase In Units.Enemies
                    unit.PowerPoint = 0
                Next


                'モラトリアムモードにする(=最終Tick時まで敵を出さない。)
                'ただし、画面に残存している敵は動いている可能性が高い。
                Me.IsMoratorium = True
                timerRestart.Interval = 1000
            Case 2
                '主人公機再配置。プレイ可能状態。(敵はまだ出ない)
                RestPlayers.Value -= 1
                Common.mainUnit = New StarA()
                Common.mainUnit.Top = Common.Display.ViewArea.Bottom - 80
                Units.Add(Common.mainUnit)
                SoundComposer.BGM1.Play()
                timerRestart.Interval = 5000
            Case 3 '第２回Tick
                'Case 3 '第３回Tick
                '通常モードへ
                Me.IsMoratorium = False
                timerRestart.Enabled = False
                restartTimerTickCount = 0
        End Select



    End Sub
End Class
