﻿Imports System.ComponentModel

Public Class UnitBase

    Public Left As Integer
    Public Top As Integer
    Public Width As Integer
    Public Height As Integer

    Public Sub New()
        Me.PowerPoint = Me.MaxPower
    End Sub

    Public Overridable Sub Move(ByVal input As InputState)
        '
    End Sub

    Public Overridable Sub Draw(ByVal g As Graphics)
        '
    End Sub

    Public Overridable Function IsDead() As Boolean
        If Me.PowerPoint > 0 Then
            Return False
        Else
            Return True
        End If
    End Function

    Public Overridable ReadOnly Property Rectangle() As Rectangle
        Get
            Return New Rectangle(Left, Top, Width, Height)
        End Get
    End Property

    Public Overridable ReadOnly Property Bottom() As Integer
        Get
            Return Top + Height
        End Get
    End Property

    Public Overridable ReadOnly Property Right() As Integer
        Get
            Return Left + Width
        End Get
    End Property

    Public Overridable Property BelongTo() As BelongTo
        Get
            Return StarAce.BelongTo.Enemy
        End Get
        Set(ByVal value As BelongTo)
            Throw New InvalidOperationException("BelongToに値を設定するには、派生クラスでBelongToをオーバーライドしてください。")
        End Set
    End Property


    ''' <summary>
    ''' 当たった時AttackPriorityが低い方が消滅します。等しいときは両方消滅します。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DefaultValue(DefaultAttackPriority)> _
    Public Overridable Property AttackPriority() As Integer
        Get
            Return m_AttackPriority
        End Get
        Set(ByVal value As Integer)
            m_AttackPriority = value
        End Set
    End Property
    Private Const DefaultAttackPriority As Integer = 10
    Private m_AttackPriority As Integer = DefaultAttackPriority


    ''' <summary>このユニットを撃破した時の得点</summary>
    Public Overridable ReadOnly Property GamePoint() As Integer
        Get
            Return 100
        End Get
    End Property

    Private m_Power As Integer
    ''' <summary>
    ''' パワー残量。これがゼロになるとこのユニットは消滅する。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Property PowerPoint() As Integer
        Get
            Return m_PowerPoint
        End Get
        Set(ByVal value As Integer)
            m_PowerPoint = value
        End Set
    End Property
    Private m_PowerPoint As Integer

    Public Overridable ReadOnly Property MaxPower() As Integer
        Get
            Return 1
        End Get
    End Property

    ''' <summary>
    ''' 被弾。damagePoint分だけPowerがひかれる。Powerがゼロになった場合Falseを返す。
    ''' </summary>
    ''' <param name="damagePoint"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Function Damage(ByVal damagePoint As Integer) As Boolean
        Me.PowerPoint -= damagePoint

        If Me.PowerPoint > 0 Then
            SoundComposer.Hit.Play()
            Return True
        Else
            Return False
        End If

    End Function

    ''' <summary>
    ''' 攻撃力。相手と接触した場合、この値分だけダメージを与える。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable ReadOnly Property AttackPoint()
        Get
            Return 1
        End Get
    End Property

    ''' <summary>
    ''' 爆発する。
    ''' ユニットの場所に爆発アニメーションを表示するだけで、ユニットの消滅自体は別途行う必要がある。
    ''' </summary>
    Public Overridable Sub Explode()
        Common.RequestUnitAdd(New ExplodeA(Me))
    End Sub

    ''' <summary>
    ''' レーザーや爆弾などの弾のときTrue。この属性がTrueのユニット同士の接触ではあ互いに消滅しない。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable ReadOnly Property IsFire() As Boolean
        Get
            Return False
        End Get
    End Property


End Class

Public Enum BelongTo
    ''' <summary>Allyに当たり判定あり。Allyからの当たり判定あり</summary>
    Enemy
    ''' <summary>Enemyに当たり判定あり。Enemyからの当たり判定なし</summary>
    Ally
    ''' <summary>当たり判定なし</summary>
    Neutral

    Item
End Enum
