﻿Public Class UnitCollection
    Inherits List(Of UnitBase)

    


    ''' <summary>
    ''' 敵ユニットの一覧です。
    ''' 現在有効な敵はすべてこの一覧に登録されています。
    ''' 撃破されたり、スクロールで画面から消え去った敵は一覧からのぞかれます。
    ''' 生成と消滅はForm1のtimerMain_Tickで行われています。
    ''' その中でUnits.AddやUnits.Removeと呼ぶと自動的に敵・味方・中立・アイテムに分けられて
    ''' Enemies, Allies, Neutrals, Itemsが追加・削除される仕組みです。
    ''' この分ける処理はこのUnitCollectionクラスのAddとRemoveの中で行っています。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Enemies() As List(Of UnitBase)
        Get
            Return m_Enemies
        End Get
        Set(ByVal value As List(Of UnitBase))
            m_Enemies = value
        End Set
    End Property
    Private m_Enemies As New List(Of UnitBase)



    ''' <summary>
    ''' 現在有効な味方の一覧です。
    ''' 主人公機と発射した弾です。
    ''' 生成と消滅はForm1のtimerMain_Tickで行われています。
    ''' その中でUnits.AddやUnits.Removeと呼ぶと自動的に敵・味方・中立・アイテムに分けられて
    ''' Enemies, Allies, Neutrals, Itemsが追加・削除される仕組みです。
    ''' この分ける処理はこのUnitCollectionクラスのAddとRemoveの中で行っています。
    ''' </summary>
    Public Property Allies() As List(Of UnitBase)
        Get
            Return m_Allies
        End Get
        Set(ByVal value As List(Of UnitBase))
            m_Allies = value
        End Set
    End Property
    Private m_Allies As New List(Of UnitBase)

    '''<summary>
    ''' 現在有効な敵でも味方でもないものの一覧です。
    ''' 表示メッセージだけに使っていると思いますが、今後背景などをユニットで表現する場合は
    ''' このNeutrals属性とすることを考えています。
    ''' 生成と消滅はForm1のtimerMain_Tickで行われています。
    ''' その中でUnits.AddやUnits.Removeと呼ぶと自動的に敵・味方・中立・アイテムに分けられて
    ''' Enemies, Allies, Neutrals, Itemsが追加・削除される仕組みです。
    ''' この分ける処理はこのUnitCollectionクラスのAddとRemoveの中で行っています。
    ''' </summary>
    Public Property Neutrals() As List(Of UnitBase)
        Get
            Return m_Neutrals
        End Get
        Set(ByVal value As List(Of UnitBase))
            m_Neutrals = value
        End Set
    End Property
    Private m_Neutrals As New List(Of UnitBase)


    '''<summary>
    ''' 現在有効なアイテムの一覧です。
    ''' 現時点ではPマークのパワーアップアイテムだけがこの一覧に属します。
    ''' 生成と消滅はForm1のtimerMain_Tickで行われています。
    ''' その中でUnits.AddやUnits.Removeと呼ぶと自動的に敵・味方・中立・アイテムに分けられて
    ''' Enemies, Allies, Neutrals, Itemsが追加・削除される仕組みです。
    ''' この分ける処理はこのUnitCollectionクラスのAddとRemoveの中で行っています。
    ''' </summary>
    Public Property Items() As List(Of UnitBase)
        Get
            Return m_Items
        End Get
        Set(ByVal value As List(Of UnitBase))
            m_Items = value
        End Set
    End Property
    Private m_Items As New List(Of UnitBase)

    Public Overloads Sub Add(ByVal item As UnitBase)
        MyBase.Add(item)

        Select Case item.BelongTo
            Case BelongTo.Ally
                Allies.Add(item)
            Case BelongTo.Enemy
                Enemies.Add(item)
            Case BelongTo.Neutral
                Neutrals.Add(item)
            Case BelongTo.Item
                Items.Add(item)
        End Select

    End Sub

    Public Overloads Sub Remove(ByVal item As UnitBase)
        MyBase.Remove(item)

        Select Case item.BelongTo
            Case BelongTo.Ally
                Allies.Remove(item)
            Case BelongTo.Enemy
                Enemies.Remove(item)
            Case BelongTo.Neutral
                Neutrals.Remove(item)
            Case BelongTo.Item
                Items.Remove(item)
        End Select
    End Sub

    Public Overloads Sub Clear()
        Enemies.Clear()
        Allies.Clear()
        Neutrals.Clear()
        Items.Clear()
        MyBase.Clear()
    End Sub

End Class
